/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
//QT Headers
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <qdom.h>

//KDE Headers
#include <kmessagebox.h>
#include <klistview.h>

//Local Headers
#include "constantsdb.h"

ConstantsDB::ConstantsDB(const QString& dbPath, KListView* list)
{
  m_constantsDatabasePath = dbPath;
  m_constantsListview = list;
}

ConstantsDB::~ConstantsDB()
{
}

/*!
    \fn ConstantsDB::readFromDatabase()
 */
void ConstantsDB::readFromDatabase()
{
  QDomDocument dbDocument;
    QFile constantsDataBaseFile(m_constantsDatabasePath);

    if(not constantsDataBaseFile.open(IO_ReadOnly)) return;

    if(not dbDocument.setContent(&constantsDataBaseFile))
      {
        constantsDataBaseFile.close();
        return;
      }
    constantsDataBaseFile.close();

    // print out the element names of all elements that are direct children
    // of the outermost element.
    QDomElement docElem = dbDocument.documentElement();

    QDomNode n = docElem.firstChild();
    while(not n.isNull())
      {
        QDomElement e = n.toElement(); // try to convert the node to an element.
        if(not e.isNull())
          {
            QListViewItem* tempItem = new QListViewItem(m_constantsListview);
            tempItem->setText(0, e.tagName());
            tempItem->setText(1, e.attribute("value"));
          }
        n = n.nextSibling();
      }
}

/*!
    \fn ConstantsDB::writeToDatabase()()
 */
void ConstantsDB::writeToDatabase()
{
  QDomDocument dbDocument;
  dbDocument.appendChild(dbDocument.createElement("constants"));

  QListViewItem* tempItem = m_constantsListview->firstChild();
  while(tempItem)
    {
      QDomElement tempElement = dbDocument.createElement(tempItem->text(0));

      tempElement.setAttribute("value", tempItem->text(1));

      dbDocument.firstChild().appendChild(tempElement);
      tempItem = tempItem->nextSibling();
    }

  QFile constantsDataBaseFile(m_constantsDatabasePath);
  if(not constantsDataBaseFile.open(IO_WriteOnly))
    KMessageBox::error(0,"Cannot store constants into database");
  else
    {
      QTextStream constantsDataBaseStream(&constantsDataBaseFile);
      constantsDataBaseStream << "<?xml version=\"1.0\" ?>\n";
      constantsDataBaseStream << dbDocument.toString(2);
      constantsDataBaseFile.close();
    }
}




