/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef FUNCTIONSDB_H
#define FUNCTIONSDB_H

//QT Headers
class QString;
#include <qmap.h>
class QDomDocument;

//KDE Headers

//Local Headers
#include "globaltypes.h"


/**
 * @short This class handles creation of user defined functions
 * @author Emiliano Gulmini<emi_barbarossa@yahoo.it>
 */
class FunctionsDB
{
  QString m_functionsDatabasePath;
  Global::LanguagesMap m_languagesMap;
  bool m_complexModeOn;
  public:
    FunctionsDB(const QString& dbPath/*, languageMap languages*/);

    ~FunctionsDB();
    /** reads the xml database of the user-defined functions */
    Global::LanguagesMap readFromDatabase();
    /** loads BC functions from database */
    Global::FunctionsCodeMap getBCCodeFromDatabase(const QDomDocument& db);
    /** loads Python functions from database */
    Global::FunctionsCodeMap getPythonCodeFromDatabase(const QDomDocument& db);
    /** creates a xml database containing the user-defined functions */
    void writeToDatabase(const Global::LanguagesMap& m);
    /** creates a xml database for user-defined BC functions */
    void putBCIntoDatabase(const Global::LanguagesMap& m, QDomDocument& db);
    /** creates a xml database for user-defined Python functions */
    void putPythonIntoDatabase(const Global::LanguagesMap& m, QDomDocument& db);
    Global::LanguagesMap functionsMap() const { return m_languagesMap; }
};

#endif
