/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

//QT Headers
#include <qtextedit.h>
#include <qregexp.h>

//KDE Headers

//Local Headers
#include "highlighter.h"

using namespace Global;

Highlighter::Highlighter(QTextEdit* t) : QSyntaxHighlighter(t)
{
}


Highlighter::~Highlighter()
{
}

/*!
    \fn Highlighter::setColor(const QColor& c, QString o)
 */
void Highlighter::setColor(const QColor& c, ColorType o) 
{
  switch(o)
    {
      case numbers: m_numbers = c; break;
      case operations: m_operations = c; break;
      case functions: m_functions = c; break;
      case keywords: m_keywords = c; break;
      case specials: m_specials = c; break;
    }
}
/*!
    \fn Highlighter::highlightParagraph(const QString & text, int endStateOfLastPara)
 */
int Highlighter::highlightParagraph(const QString & text, int endStateOfLastPara)
{
  endStateOfLastPara = 0;

  setFormat(0 , text.length(), QColor(0, 0, 0));

  applyHighlighting(text, "[0123456789ABCDEF]+" , m_numbers);
  applyHighlighting(text, "[\\*/\\+\\-%<>=!]+" , m_operations);
  applyHighlighting(text, "[a-z]+", m_functions);
  applyHighlighting(text, "if|else|while|for|break|continue|halt|return|print|and|elif|continue|lambda|or|pass", m_keywords);

  return endStateOfLastPara;
}

void Highlighter::applyHighlighting(const QString& text ,const QString& expr, const QColor& c)
{
  QRegExp exp(expr);
  int pos = 0;
  pos = exp.search(text, pos);

  while(pos > -1)
    {
      uint mlength = exp.matchedLength();
      setFormat(pos, mlength, c);
      pos = exp.search(text, pos + mlength);
    }
}

