/***************************************************************************
 *   Copyright (C) 2005 by Emiliano Gulmini   *
 *   emi_barbarossa@yahoo.it   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#ifndef _KALCOOLUS_H_
#define _KALCOOLUS_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

//QT Headers
class QFile;
class QFont;
class QColor;
//KDE Headers
class KConfig;
class KProcess;
//Local Headers
#include "kbc.h"
class ConstantsDB;
class FunctionsDB;
class Configuration;
class Highlighter;
class KalcoolusImplementIf;
#include "globaltypes.h"
/**
 * @short Application Main Window
 * @author Emiliano Gulmini <emi_barbarossa@yahoo.it>
 * @version 0.1.8
 */
class Kalcoolus : public Kbc
{
    Q_OBJECT
public:
  /**
    * Default Constructor
    */
  Kalcoolus();

  /**
   * Default Destructor
   */
  virtual ~Kalcoolus();

private:
  QString m_inputBuffer,
          m_resultBuffer,
          m_last;
  Global::LanguagesMap m_languagesMap;
  Global::ConstantsMap m_constantsMap;
  QMap<QString, QString> m_buttonsMap;
  KConfig* m_config;
  QString m_constantsDatabasePath,
          m_functionsDatabasePath;
  QString m_program,
          m_bcScriptFilePath,
          m_pythonScriptFilePath;
  ConstantsDB* m_constantsDB;
  FunctionsDB* m_functionsDB;
  Configuration* m_configuration;
  Highlighter* m_inputHighlighter,
             * m_resultHighlighter,
             * m_functionEditorHighlighter;
  QString m_colorNumbers,
          m_colorOperations,
          m_colorFunctions,
          m_colorKeywords;
KalcoolusImplementIf* m_interface;
private:
  int min(int a, int b) { return (a < b ? a : b); }
  /** Group the connect() in the contructor */
  void ConnectKeyboardPage();
  void ConnectConstantsPage();
  void ConnectFunctionsPage();
  void ConnectConfigurationPage();
  /** Maps keyboard buttons */
  void mapButtons();
  /** Reading/writing constants tab page */
  void loadConstantsIntoBox();
  /** Reading/writing functions tab page */
  void loadFunctionsIntoBoxes();
  /** Reading/writing configuration tab page */
  void loadBCSettings();
  void writeBCSettings();
  void loadFontSettings();
  void writeFontSettings();
  void loadPythonSettings();
  void writePythonSettings();
  void loadOptionsSettings();
  void writeOptionsSettings();
  /** Enables/disables buttons on setting events*/
  void activateButtons();
  /** Create help */
  void createHelp();
  /** Wrap functions */
  void setNumberButtonsColor(const QColor& c);
  void setOperationButtonsColor(const QColor& c);
  void setFunctionButtonsColor(const QColor& c);
  void setKeywordButtonsColor(const QColor& c);
  void setSpecialButtonsColor(const QColor& c);
  void setGUIColor(QString color, Global::ColorType t);
  void insertStringIntoDisplay(const QString& s, int dpara, int dindex);
private slots:
  /** Keyboard page */
  void slotNumberButton();
  void slotOperatorButton();
  void slotLastResultButton();
  void slotEqualClicked();
  void slotEraseClicked();
  void slotCancelClicked();
  void slotCopyrights();
  void slotNewLineClicked();
  void slotFunctionButton();
  void slotConstantsBox(const QString & s);
  void slotBye();
  void slotSave();
  void slotComplexMode();
  void slotImaginaryButton();
  /** Constants page */
  void slotAddConstant();
  void slotClearConstant();
  void slotConstSelected(QListViewItem* item);
  /** Functions page */
  void slotAddUserFunction();
  void slotDeleteUserFunction();
  void slotEditUserFunction(int index);
  void slotSelectUserFunction(const QString& s);
  /** Configuration page */
  void slotScale(const QString& s);
  void slotIBase(const QString& s);
  void slotOBase(const QString& s);
  void slotPyDecimalDigits(const QString& s);
  void slotChooseConfigItem(QIconViewItem* item);
  void slotFontSelected(const QFont& f);
  void slotColorNumbers(const QColor& c);
  void slotColorOperations(const QColor& c);
  void slotColorFunctions(const QColor& c);
  void slotColorKeywords(const QColor& c);
  void slotColorSpecial(const QColor& c);
  void slotEnableLogFile(bool b);
  void slotHelp();
  void slotGetScriptError(KProcess* proc, char* s, int i);
  void slotGetScriptOutput(KProcess* proc, char* s, int i);
  void slotProcessExited(KProcess* proc);
};

#endif // _KALCOOLUS_H_
